import tensorrt as trt
from int8_custom_calibrator import Int8Calibrator
import os

ONNX_FILE_PATH = 'resnet50_pth_to_onnx.onnx'
# logger to capture errors, warnings, and other information during the build and inference phases
TRT_LOGGER = trt.Logger()

PRECISION = 'INT8'

def build_engine(onnx_file_path,Int8_calibrator):
    # initialize TensorRT engine and parse ONNX model
    builder = trt.Builder(TRT_LOGGER)
    network = builder.create_network(1 << int(trt.NetworkDefinitionCreationFlag.EXPLICIT_BATCH))
    parser = trt.OnnxParser(network, TRT_LOGGER)

    # allow TensorRT to use up to 1GB of GPU memory for tactic selection
    #builder.max_workspace_size = 1 << 30
    # we have only one image in batch
    builder.max_batch_size = 1

    # Creating execution context
    config = builder.create_builder_config()
    #config.set_memory_pool_limit(trt.MemoryPoolType.WORKSPACE, 1 << 20)
    config.profiling_verbosity = trt.ProfilingVerbosity.DETAILED

    # Calibration cofig
    if builder.platform_has_fast_int8:
        print('Yes! Continuing in INT8 mode')
        config.set_flag(trt.BuilderFlag.INT8)
        config.int8_calibrator = Int8_calibrator
    else:
        exit  

    # parse ONNX
    with open(onnx_file_path, 'rb') as model:
        print('Beginning ONNX file parsing')
        parser.parse(model.read())
    print('Completed parsing of ONNX file')

    # generate TensorRT engine optimized for the target platform
    print('Building an engine...')
    engine = builder.build_serialized_network(network, config)

    with open('INT8/resnet50_pth_onnc_trt_'+PRECISION+'.engine', 'wb') as f:
        f.write(engine)

    print("Completed creating Engine")
    print('******************************')

    return engine


def main():
    # initialize TensorRT engine and parse ONNX model
    print('******************************')
    print('Started building engine...')

    cache_file = 'resnet50_int8_calibration.cache'
    training_set = 'imagenet/imagenet_images/'
    img_per_batch = 5
    #batchstream = ImageBatchStream(NUM_IMAGES_PER_BATCH, calibration_files)
    Int8_calibrator = Int8Calibrator(training_set, cache_file=cache_file, batch_size=img_per_batch)
    engine = build_engine(ONNX_FILE_PATH,Int8_calibrator)


if __name__ == '__main__':
    main()
